/***************************************************************************
 *   Copyright (C) 2009 by Rene Landert <rene.landert@bluewin.ch>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************
 *                                                                         *
 *   Part of the code in this file was taken from KDE4Powersave and/or     *
 *   Lithium, where noticed.                                               *
 *                                                                         *
 **************************************************************************/

#include "KchildlockDaemon.h"

#include <kdebug.h>
#include <kuser.h>





// do the monitoring of usage for a given user
// ===========================================
void KchildlockDaemon::monitor_user()
{
  QDate actdate;
  QTime acttime;

//  if (DBus_conn_available == false) return;                     // no console kit
  if (current_user->get_user() == "") return;	        // no session active
  actdate = QDate::currentDate();
  acttime = QTime::currentTime();
  if (my_consumption->my_cons_record.pcu.weekofyear != actdate.weekNumber())
     {
     // its a new week, reset all daily/weekly working set values to zero
     my_consumption->set_default_values(current_user->get_user());
     }
  if (my_consumption->my_cons_record.pcu.dayofweek != actdate.dayOfWeek())
      {
      // its a new day, reset all daily working set values to zero
      my_consumption->reset_daily_values();
      current_user->set_warn_usr_state(KCHILDLOCK_NOWARNSTATE);
      current_user->set_warn_app_state(KCHILDLOCK_NOWARNSTATE);
      }

   if (debugflag) {
      fprintf (debugfile,"function=monitor_user user=%s at %i.%i.%i %i:%i:%i\n", current_user->get_user().toAscii().constData(),
               actdate.day(), actdate.month(), actdate.year(), acttime.hour(), acttime.minute(), acttime.second());
      fflush(debugfile);
      }

// check for pc usage limits and for application usage limits
     current_limits->set_nolimits(TRUE);
     current_limits->get_current_userlimits (current_user->get_user(), &myUserSettingsKCG, &myGroupSettingsKCG, UserSettings_filename, GroupSettings_filename, my_consumption);

     if (!current_limits->get_nolimits()) {
        check_user_against_limits(current_limits, current_user->get_user()
        );
     }
}





// get the current active KDE User
// ===============================
QString KchildlockDaemon::get_active_user()
{

/* Just to remember:  the command line is:
gdbus call --system --dest org.freedesktop.login1 --object-path /org/freedesktop/login1 --method org.freedesktop.login1.Manager.ListSessions 
Answer:   
([('1', uint32 1000, 'rene', 'seat0', objectpath '/org/freedesktop/login1/session/_31'), ('17', 1002, 'kdestd', 'seat0', '/org/freedesktop/login1/session/_317')],)


gdbus call --system --dest org.freedesktop.login1 
  --object-path /org/freedesktop/login1/user/_1000 
  --method org.freedesktop.DBus.Properties.Get org.freedesktop.login1.User State

answer is:  (<'active'>,)


gdbus call --system --dest org.freedesktop.login1 
  --object-path /org/freedesktop/login1/user/_1000 
  --method org.freedesktop.DBus.Properties.Get org.freedesktop.login1.User Name

answer is:  (<'rene'>,)

*/
   int userid;
   int ctr;
   QString m_name, m_path, m_actusr;
   int m_uid;
   
   QDBusConnection m_bus = QDBusConnection::systemBus();
   QDBusInterface login1_iface("org.freedesktop.login1", "/org/freedesktop/login1", "org.freedesktop.login1.Manager", m_bus);
   QDBusMessage replylist1 = login1_iface.call("ListSessions");
   if ((replylist1.type() == QDBusMessage::ErrorMessage) || (replylist1.type() == QDBusMessage::InvalidMessage)) {
//        kError() << "KCHILDLOCK: The session is not registered with login1";
       userid = 0;
       m_actusr="";
       return(m_actusr);
   }

   userid = 0;
   m_actusr="";

//	qDebug() << "KCH: arg.usertype " << replylist1.arguments() [0].userType();
   QString my_debugbfr= QString::null;
   QDebug my_debuginfo = QDebug(&my_debugbfr);
   my_debuginfo << replylist1;
//	qDebug() << my_debugbfr;
   
   
// analyse the replystring. The format for 2 concurent sessions is:
//    " QDBusMessage(type=MethodReturn, service=":1.2", signature="a(susso)", contents=([Argument: a(susso) 
//    {[Argument: (susso) "1", 1000, "rene", "seat0", [ObjectPath: /org/freedesktop/login1/session/_31]], 
//    [Argument: (susso) "4", 1002, "kdestd", "seat0", [ObjectPath: /org/freedesktop/login1/session/_34]]}]) ) " 
// we need the path string!!

   bool atend = false;
   int position;
   int count = 1;
   QString my_substring0, my_substring1, my_substring2, my_substring3, my_substring9;
   position = my_debugbfr.indexOf("{", 0);
   my_substring0 = my_debugbfr.remove(0, position+1);
   my_substring1 = my_substring0;


   while (atend == false) {
     if ((my_substring0.indexOf("[ObjectPath:", 0))  == -1) {
        userid = 0;
        m_actusr="";
	atend = true;
	return(m_actusr);
     } else {
     position = (my_substring0.indexOf("[ObjectPath:",0)) + 12 ;
     my_substring2 = my_substring0.remove(0, position+1);
     position = my_substring2.indexOf("]", 0);
     m_path =my_substring2.remove(position, my_substring0.size());
     m_path = m_path.replace("session", "user");
     position = (m_path.indexOf("_", 0) +1);
     m_path = m_path.remove(position, m_path.size() + 3);
     
     position = my_substring1.indexOf("rgument", 0);
     my_substring3 = my_substring1.remove(0, position + 8);
     position = my_substring3.indexOf(",", 0);
     my_substring3 = my_substring3.remove(0, position + 2);
     position = (my_substring3.indexOf(" ", 1)) - 1 ;
     my_substring3 = my_substring3.remove(position, my_substring3.size());
     m_path = m_path.append(my_substring3);

     m_bus = QDBusConnection::systemBus();
     QDBusInterface session_iface("org.freedesktop.login1", m_path, "org.freedesktop.DBus.Properties", m_bus);
     QDBusMessage reply2 = session_iface.call ("Get", "org.freedesktop.login1.User", "State");
     QDBusMessage reply3 = session_iface.call ("Get", "org.freedesktop.login1.User", "Name");

     // answer is: QDBusMessage(type=MethodReturn, service=":1.2", signature="v", contents=([Variant(QString): "rene"]) ) 
     // and:       QDBusMessage(type=MethodReturn, service=":1.2", signature="v", contents=([Variant(QString): "active"]) ) 

     if ((reply2.type() == QDBusMessage::ErrorMessage) || (reply2.type() == QDBusMessage::InvalidMessage)  || 
        (reply3.type() == QDBusMessage::ErrorMessage) || (reply3.type() == QDBusMessage::InvalidMessage) ) {
//        kError() << "KCHILDLOCK: The sessioninformation Name and State are not registered with login1";
        userid = 0;
        m_actusr="";
        return(m_actusr);
     }
     my_debugbfr= QString::null;
     QDebug my_debuginfo = QDebug(&my_debugbfr);
     my_debuginfo << reply2;

     position = my_debugbfr.indexOf("[Var", 0);
     my_substring9 = my_debugbfr.remove(0, position);
     position = my_substring9.indexOf("):", 0);
     my_substring9 = my_debugbfr.remove(0, position+4);

     position = my_substring9.indexOf("]", 0);
     my_substring9 = my_substring9.remove(position-1, my_substring9.size() + 2);
     
     if (my_substring9 == "active") {
       atend = true;
       my_debugbfr= QString::null;
       QDebug my_debuginfo = QDebug(&my_debugbfr);
       my_debuginfo << reply3;

       position = my_debugbfr.indexOf("[Var", 0);
       my_substring9 = my_debugbfr.remove(0, position);
       position = my_substring9.indexOf("):", 0);
       my_substring9 = my_debugbfr.remove(0, position+4);

       position = my_substring9.indexOf("]", 0);
       my_substring9 = my_substring9.remove(position-1, my_substring9.size() + 2);

       m_actusr = my_substring9;
       
     };
    
     };
     
   };
   return(m_actusr);
}

